﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DataSetClient;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.Utilities;
using Produmex.Sbo.Logex.Data.Generators.LocationController;
using System.Data.SqlClient;
using System.Transactions;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Sbo.Logex.Data;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_ExpeditionEntryScriptAL
{
    /// <summary>
    /// Expedition entry
    /// </summary>
    public class WorkflowScript_ExpeditionEntryScriptAL : WorkflowInstanceScriptBase
    {
        /// <summary>
        /// Definitions
        /// </summary>
        private static class Definitions
        {
            /// <summary>
            /// The separator for separating strings values
            /// </summary>
            public const string SEPARATOR = ", ";

            /// <summary>
            /// Minimum number of characters when entering reason
            /// </summary>
            public const int MIMIMUM_NUMBER_OF_CHARACTERS_FOR_REASON = 10;


            //report definitions
            public const string REPORT_PATH_LABEL = "ContainerDocument.rpt";
            public const int NUMBER_OF_LABELS = 1;
        }

        // Input parameters
        /// <summary>
        /// The default culture information
        /// </summary>
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        /// <summary>
        /// The PMX ose company
        /// </summary>
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        /// <summary>
        /// The cross docking
        /// </summary>
        public ReadOnlyBinder<bool> CrossDocking;

        /// <summary>
        /// Gets the current_ default culture information.
        /// </summary>
        /// <value>
        /// The current_ default culture information.
        /// </value>
        private CultureInfo current_DefaultCultureInfo
        {
            get { return DefaultCultureInfo.Get(); }
        }

        /// <summary>
        /// Gets the current PMX ose company.
        /// </summary>
        /// <value>
        /// The current PMX ose company.
        /// </value>
        private PmxOseCompany currentPmxOseCompany
        {
            get { return PmxOseCompany.Get(); }
        }

        //Subflows
        /// <summary>
        /// The m_enter quantity sub flow
        /// </summary>
        WorkflowInstanceBase m_enterQuantitySubFlow;

        /// <summary>
        /// The m_sub flow back requested
        /// </summary>
        protected bool m_subFlowBackRequested;
        /// <summary>
        /// The m_sub flow list of serial numbers
        /// </summary>
        protected Collection<string> m_subFlowListOfSerialNumbers;
        /// <summary>
        /// The m_sub flow quantity
        /// </summary>
        protected double m_subFlowQuantity;

        private PmxContainer container;

        /// <summary>
        /// The m_print item label flow
        /// </summary>
        private WorkflowInstanceBase m_printItemLabelFlow;
        /// <summary>
        /// The m_print item label flow_ back requested
        /// </summary>
        private bool m_printItemLabelFlow_BackRequested = false;

        /// <summary>
        /// The m_print item label flow_ location code
        /// </summary>
        private string m_printItemLabelFlow_LocationCode;
        /// <summary>
        /// The m_print item label flow_ needed item code
        /// </summary>
        private string m_printItemLabelFlow_NeededItemCode;
        /// <summary>
        /// The m_print item label flow_ needed packaging type
        /// </summary>
        private string m_printItemLabelFlow_NeededPackagingType;

        private int? m_printItemLabelFlow_NeededQuantityByPackagingType;


        /// <summary>
        /// The m_print item label flow_ needed batch number
        /// </summary>
        private string m_printItemLabelFlow_NeededBatchNumber;
        /// <summary>
        /// The m_print item label flow_ needed batch number2
        /// </summary>
        private string m_printItemLabelFlow_NeededBatchNumber2;
        /// <summary>
        /// The m_print item label flow_ needed best before date
        /// </summary>
        private DateTime? m_printItemLabelFlow_NeededBestBeforeDate;
        /// <summary>
        /// The m_print item label flow_ needed nr of labels
        /// </summary>
        private int? m_printItemLabelFlow_NeededNrOfLabels;
        /// <summary>
        /// The m_print item label flow_ entered quantity
        /// </summary>
        private double? m_printItemLabelFlow_EnteredQuantity;

        private double? m_printItemLabelFlow_EnteredQuantityUom2;
        private bool m_printItemLabelFlow_ShowPrintMessage;
        private DataTable m_currentContainerLines;
        /// <summary>
        /// The m_enter batch attributes flow
        /// </summary>
        private WorkflowInstanceBase m_enterBatchAttributesFlow;
        /// <summary>
        /// The m_enter batch attributes flow_ list of batch attributes
        /// </summary>
        protected Collection<ItemTransactionalBatchAttributeInfo> m_enterBatchAttributesFlow_ListOfBatchAttributes;

        /// <summary>
        /// The m_enter quality values flow
        /// </summary>
        private WorkflowInstanceBase m_enterQualityValuesFlow;

        /// <summary>
        /// The m_enter quality values flow_ list of quality values
        /// </summary>
        protected Collection<QualityValueInfo> m_enterQualityValuesFlow_ListOfQualityValues;

        /// <summary>
        /// The s_log
        /// </summary>
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        private CultureInfo m_defaultCultureInfo;

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_ExpeditionEntryScriptAL"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_ExpeditionEntryScriptAL(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
            m_enterQuantitySubFlow = this.Create("EnterQuantityScript");
            m_enterBatchAttributesFlow = this.Create("EnterBatchAttributesScript");
            m_enterQualityValuesFlow = this.Create("EnterQualityValuesScript");

        }

        #endregion

        #region WorkflowInstanceScriptBase Members

        /// <summary>
        /// Executes this instance.
        /// </summary>
        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
            PmxBusinessPartnerInfo businessPartnerInfo = null;

            string initialErrorKey = null;
            string deviceID = session.DeviceInstanceId;
            SboUserInfo userInfo = sboProviderService.GetCurrentUser();
            Message msg = null;
            string query = null;
            PmxOseWarehouseInfo whsInfo = null;
            string currentDockCode = null;
            string cardCode = null;
            DataSet dsSuppliers = null;
            PmxItemInfo itemInfo = null;
            string gtin = null;
            Collection<LogisticUnitGoodsReceipt> purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceipt>();
            string secondBatchNumber = null;
            DateTime? bestBeforeDate = null;
            double quantity = 0;
            DataSet dsExistingBatch = null;

            int? numberOfItemLabels = 0;
            bool receiveAllItemsOnOnePurchaseDelivery = false;
            double? numberOfItemsInPurchaseUnit = null;
            string purchaseUom = null;
            List<string> allowedButtonsIdentifyProductScreen = null;
            bool goBack = false;

            Collection<string> listOfSerialNumbers = new Collection<string>();

            bool allowLabelsToBeScannedOnVendor = false;
            bool allowLogisticCarriersOnVendor = false;

            DataSet dsReasons = null;

            string reasonCode = null;
            string reasonName = null;
            List<string> batchNumber1ToCheck = new List<string>();
            List<string> batchNumber2ToCheck = new List<string>();
            List<DateTime?> bestBeforeDatesToCheck = new List<DateTime?>();
            List<string> serialNumbersToCheck = new List<string>();

            whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), deviceID);

            receiveAllItemsOnOnePurchaseDelivery = sboProviderService.InvokeMethodWithDbConnection<bool>(true, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteGetReceiveAllItemsOnOnePurchaseDelivery));

            bool allowToSelectBatch = sboProviderService.InvokeMethodWithDbConnection<bool>(true, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteGetAllowToSelectBatches));
            bool crossDocking = this.CrossDocking != null && this.CrossDocking.Get();

            IBarcodeController barcodeController = null;

            bool autoSelectInternalBatch = false;
            bool allowToEnterReason = false;
            bool splitQtyInLogUnitsOfItemDefQtyOnLogUnit = false;
            bool checkDataInUDFPOLine = false;
            bool reserveInvoicesAreAllowed = false;

            Collection<string> listOfLocationsForWA = new Collection<string>();

            sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                delegate (PmxDbConnection conn, object[] parameters)
                {
                    PmxContainerProvider cntprov = new PmxContainerProvider(conn);
                    Dictionary<string, PmxParameterValue> ctnparams = cntprov.GetParametersContainerManagementController();

                    reserveInvoicesAreAllowed = PmxParameterValueProvider.GetParameterValue<bool>(ctnparams, PmxExtensionParameterCodes.CONTAINER_MANAGEMENT_CONTROLLER_PURCHASE_RESERVE_INVOICES_ALLOWED, false);



                    PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
                    barcodeController = bcProv.GetBarcodeController();

                    PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider();
                    Dictionary<string, PmxParameterValue> pdParams = prov.GetParametersPurchaseOrderGenerator(conn);

                    autoSelectInternalBatch = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_AUTO_SELECT_INTERNAL_BATCH, false);
                    allowToEnterReason = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_ENTER_REASON_BY_ITEM, false);
                    splitQtyInLogUnitsOfItemDefQtyOnLogUnit = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_SPLIT_QTY_IN_LOG_UNITS_OF_ITEM_DEF_QTY_ON_LOG_UNIT, false);
                    checkDataInUDFPOLine = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_CHECK_DATA_FROM_PO, false);

                    PmxOseWarehouseAutomationProvider waProv = new PmxOseWarehouseAutomationProvider(conn);
                    listOfLocationsForWA = waProv.GetAllInputLocationsForWA(true);

                    return null;
                }));





            //initialize sub flows
            m_printItemLabelFlow = Create("PrintItemLabelScript");
            m_printItemLabelFlow.BindParameters(
                this.DefaultCultureInfo,
                this.PmxOseCompany,
                new ReadOnlyBinder<string>("LocationCode", this, "m_printItemLabelFlow_LocationCode"),
                new ReadOnlyBinder<string>("NeededItemCode", this, "m_printItemLabelFlow_NeededItemCode"),
                new ReadOnlyBinder<string>("NeededPackagingType", this, "m_printItemLabelFlow_NeededPackagingType"),
                new ReadOnlyBinder<int?>("NeededQuantityByPackagingType", this, "m_printItemLabelFlow_NeededQuantityByPackagingType"),
                new ReadOnlyBinder<string>("NeededBatchNumber", this, "m_printItemLabelFlow_NeededBatchNumber"),
                new ReadOnlyBinder<string>("NeededBatchNumber2", this, "m_printItemLabelFlow_NeededBatchNumber2"),
                new ReadOnlyBinder<DateTime?>("NeededBestBeforeDate", this, "m_printItemLabelFlow_NeededBestBeforeDate"),
                new ReadOnlyBinder<int?>("NeededNrOfLabels", this, "m_printItemLabelFlow_NeededNrOfLabels"),
                new ReadWriteBinder<bool>("BackRequested", this, "m_printItemLabelFlow_BackRequested"),
                new ReadWriteBinder<double?>("EnteredQuantity", this, "m_printItemLabelFlow_EnteredQuantity"),
                new ReadWriteBinder<double?>("EnteredQuantityUom2", this, "m_printItemLabelFlow_EnteredQuantityUom2"),
                new ReadWriteBinder<bool>("ShowPrintMessage", this, "m_printItemLabelFlow_ShowPrintMessage"));


            bool receiveGoodsOnSelectedLocation = false;
            bool showReserveInvoices = false;
            bool allowToExceedOrderedQuantity = true;
            PmxParameterValue crossDockingPickListType = null;

            sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null, new DelegateWithPmxDbConnection<object>(
                delegate (PmxDbConnection conn, object[] parameters)
                {
                    PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider((SboConnection)conn);
                    Dictionary<string, PmxParameterValue> parms = prov.GetParametersPurchaseOrderGenerator(conn);

                    receiveGoodsOnSelectedLocation = PmxParameterValueProvider.GetParameterValue<bool>(parms, PmxExtensionParameterCodes.PURCHASE_DELIVERY_RECEIVE_ON_LOCATION, false);
                    showReserveInvoices = PmxParameterValueProvider.GetParameterValue<bool>(parms, PmxExtensionParameterCodes.PURCHASE_DELIVERY_SHOW_RESERVE_INVOICES, false);
                    allowToExceedOrderedQuantity = PmxParameterValueProvider.GetParameterValue<bool>(parms, PmxExtensionParameterCodes.PURCHASE_DELIVERY_ALLOW_TO_EXCEED_ORDERED_QTY, true);
                    if (crossDocking)
                    {
                        parms.TryGetValue(PmxExtensionParameterCodes.PURCHASE_DELIVERY_CROSS_DOCKING_PICKLISTTYPE, out crossDockingPickListType);
                    }

                    return null;
                }));

            bool askForWeight = false;
            double? weight = null;
            Dictionary<string, PmxParameterValue> parameter = new Dictionary<string, PmxParameterValue>();
            Dictionary<string, PmxItemInfo> listOfItemInfos = new Dictionary<string, PmxItemInfo>();
            double currentPrice = 0;
            double numOfPackages = 0;

            SboCompanyAdminInfo sboCompanyAdminInfo = null;
            sboCompanyAdminInfo = sboProviderService.InvokeMethodWithDbConnection<SboCompanyAdminInfo>(true, false, null, null, new DelegateWithPmxDbConnection<SboCompanyAdminInfo>(ExecuteGetSboCompanyInfo));


            #region LOCATION SUGGESTIONS

            bool useLocationSuggestions = false;
            Collection<LocationInfo> listOfLocationInfos = null;
            Dictionary<string, LocationInfo> listOfLocationInfosAsDictionary = null;
            bool needsToEnterReasonForNotAllowedLocation = false;
            string reasonCodeLocations = null;
            string reasonNameLocations = null;
            string reasonFreeTextLocations = null;
            string reasonLocationCode = null;

            Collection<string> listOfUsedLocations = null;

        #endregion


        Step_SelectSupplier:

            #region SELECT SUPPLIER

            cardCode = null;

            query = BuildQuery.GetSuppliersOrderByName();
            dsSuppliers = sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.ISelectSupplierScreen),
                 DefaultCultureInfo.Get(), BuildParamCollection(
                 "InitialErrorKey", initialErrorKey,
                 "TitleKey", "MSG_TITLE_SELECT_SUPPLIER",
                 "SupplierDS", dsSuppliers
                 ));

            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;
            if (msg.Name.EndsWith(".SupplierSelected"))
            {
                cardCode = ExtractParameter<string>(msg.Parameters, "supplierCode");

                businessPartnerInfo = sboProviderService.InvokeMethodWithDbConnection<PmxBusinessPartnerInfo>(true, false, null, null, new DelegateWithPmxDbConnection<PmxBusinessPartnerInfo>(ExecuteGetBusinessPartnerInfo), cardCode);

                allowLabelsToBeScannedOnVendor = !businessPartnerInfo.HasNoLabelsAtReception;
                allowLogisticCarriersOnVendor = !businessPartnerInfo.HasNoLogisticCarriersAtReception;

                goto Step_EnterNumberOfPackages;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;

                return;
            }

            goto Step_SelectSupplier;

        #endregion

        Step_EnterNumberOfPackages:

            #region ENTERNUMBEROFPACKAGES

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_ENTER_QUANTITY_OF_PACKAGES",
                    "InitialQuantity", 1,
                    "NumberOfDigitsAfterDecimal", 0,
                    "MaximumQuantity", -1,
                    "SetFocusToQuantityControl", true
                   ));

            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;

            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                numOfPackages = ExtractParameter<double>(msg.Parameters, "quantity");

                goto Step_EnterShippingType;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;

                goto Step_SelectSupplier;
            }
            goto Step_EnterNumberOfPackages;

        #endregion

        Step_EnterShippingType:

            #region ENTERSHIPPINGTYPE

            DataSet dsShippingTypes = null;
            string shippingType = null;

            query = BuildQuery.GetAllShippingType(sboProviderService.GetDbTool());
            dsShippingTypes = sboProviderService.RunView(false, null, null, query);

            if (dsShippingTypes.Tables[0].Rows.Count > 0)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectCustomerScreen),
                    this.DefaultCultureInfo, BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_SHIPPING_TYPE",
                    "CustomerDS", dsShippingTypes
                    ));
                msg = WaitForMessage();
                initialErrorKey = null;

                if (msg.Name.EndsWith(".CustomerSelected"))
                {
                    shippingType = ExtractParameter<string>(msg.Parameters, "customerCode");

                    goto Step_EnterExpeditionNumber;
                }
                else if (msg.Name.EndsWith(".BackRequested"))
                {
                    goBack = true;

                    goto Step_EnterNumberOfPackages;
                }
            }

            goto Step_EnterShippingType;

        #endregion

        Step_EnterExpeditionNumber:

            #region ENTEREXPEDITIONNUMBER

            string expeditionNumber;
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                     DefaultCultureInfo.Get(), BuildParamCollection(
                         "InitialErrorKey", initialErrorKey,
                         "TitleKey", "MSG_TITLE_ENTER_EXPEDITION_NUMBER"
                         //, "AllowToGoBack", true,
                         //"EnabledToolBarButtons", ToolBarButtons.Back | ToolBarButtons.Forward //| ToolBarButtons.Time
                         ));

            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;

            if (msg.Name.EndsWith(".StringEntered"))
            {
                expeditionNumber = ExtractParameter<string>(msg.Parameters, "stringValue");

                goto Step_EnterWeight;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_EnterShippingType;
            }

            goto Step_EnterExpeditionNumber;

        #endregion

        Step_EnterWeight:

            #region ENTER WEIGHT

            //Get master data
            int weightUomID = sboCompanyAdminInfo.StandardUnitOfWeight;
            int numberOfDecimals = sboCompanyAdminInfo.MeasuringDecimals;
            int unitCode = sboCompanyAdminInfo.StandardUnitOfLength;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
               "InitialErrorKey", initialErrorKey,
               "TitleKey", "MSG_TITLE_ENTER_WEIGHT",
               "InitialQuantity", 0,
               "NumberOfDigitsAfterDecimal", numberOfDecimals,
               "MaximumQuantity", -1
               ));
            msg = WaitForMessage();

            initialErrorKey = null;


            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                weight = ExtractParameter<double>(msg.Parameters, "quantity");

                goto Step_EnterPrice;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                //not allowed to go back
                //initialErrorKey = "MSG_ERROR_CANNOT_GO_BACK";
                goto Step_EnterExpeditionNumber;
            }

            //}

            goto Step_EnterWeight;

        #endregion

        Step_EnterPrice:

            #region ENTER PRICE
            string currency = sboCompanyAdminInfo.LocalCurrency;
            numberOfDecimals = sboCompanyAdminInfo.PriceDecimals;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                   "InitialErrorKey", initialErrorKey,
                   "TitleKey", "MSG_TITLE_ENTER_PRICE",
                   "InitialQuantity", 0,
                   "NumberOfDigitsAfterDecimal", 2,
                   "MaximumQuantity", -1
                   ));
            msg = WaitForMessage();


            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                currentPrice = ExtractParameter<double>(msg.Parameters, "quantity");

                goto Step_Comments;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_EnterWeight;
            }

            goto Step_EnterPrice;

        #endregion

        Step_Comments:

            #region COMMENTS

            string comment;
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                    m_defaultCultureInfo, BuildParamCollection(
                   "InitialErrorKey", initialErrorKey,
                   "TitleKey", "MSG_TITLE_ENTER_COMMENT",
                   "AllowToGoBack", true,
                   "AllowMultiLine", true
                   ));
            msg = WaitForMessage();

            if (msg.Name.EndsWith(".StringEntered"))
            {
                comment = ExtractParameter<string>(msg.Parameters, "stringValue");

                goto Step_AskReason;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_EnterPrice;
            }

            goto Step_Comments;

        #endregion

        Step_AskReason:

            #region ASK REASON

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "TitleKey", "MSG_TITLE_ASK_REASON",
                                "MessageKey", "MSG_CONFIRM_REASON"));
            msg = WaitForMessage();

            if (msg.Name.EndsWith("IDecisionScreenReply.Yes"))
            {
                goto Step_ReasonScreen;
            }
            else if (msg.Name.EndsWith("IDecisionScreenReply.No"))
            {
                reasonCode = null;

                goto Step_CreateContainer;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_Comments;
            }

            goto Step_AskReason;

        #endregion

        Step_ReasonScreen:

            #region REASON SCREEN

            query = BuildQuery.GetReasons();
            dsReasons = sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectReasonScreen),
                this.DefaultCultureInfo, BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_REASON",
                //"MessageKey", "MSG_SELECT_SHIPPING_REASON",
                "ReasonDS", dsReasons));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (!msg.Name.EndsWith(".ReasonSelected"))
            {
                goto Step_Comments;
            }

            reasonCode = ExtractParameter<string>(msg.Parameters, "reasonCode");

            DataRow[] temp_rows = dsReasons.Tables[0].Select(String.Format("{0} = '{1}'", ReasonDataSet.Columns.REASON_CODE.Name, reasonCode));
            if (temp_rows.Length > 0)
            {
                reasonName = temp_rows[0][ReasonDataSet.Columns.REASON_NAME.Name] as string;
            }

            /*#region ENTER REASON FREE TEXT

            if (reasonsCodesRequiresFreeText.ContainsKey(reasonCode) &&
                reasonsCodesRequiresFreeText[reasonCode])
            {
                // Show screen with list
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_ENTER_REASON",
                        "Information", "MSG_ENTER_FREE_REASON_DEFAULT",
                        "Parameters", new object[] { reasonName },
                        "AllowToGoBack", true,
                        "ForceDataEntry", true,
                        "AllowMultiLine", true,
                        "MinimumNumberOfCharacters", Definitions.MIMIMUM_NUMBER_OF_CHARACTERS_FOR_REASON
                        ));

                msg = WaitForMessage();

                initialErrorKey = null;

                if (msg.Name.EndsWith(".StringEntered"))
                {
                    reasonFreeText = ExtractParameter<string>(msg.Parameters, "stringValue");

                    goto Step_CreateContainer;
                }
                else
                {
                    //go back
                    goto Step_Comments;
                }
            }
            #endregion*/

            goto Step_CreateContainer;

        #endregion

        Step_CreateContainer:

            #region CREATECONTAINER

            


            sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
              delegate (PmxDbConnection conn, object[] parameters)
              {
                  string containerCode = DateTime.Now.ToString("yyyyMMddHHmmss");

                  if (!string.IsNullOrEmpty(comment))
                  {
                      comment += " - ";
                  }

                  comment += string.Format("{0}({1})", sboProviderService.GetCurrentUser().Username, conn.InternalKeySboUser);

                  PmxContainerProvider cprov = new PmxContainerProvider(conn);
                  container = cprov.GetNewBO();
                  container.CardCode = cardCode;
                  container.ContainerCode = comment;
                  container.ContainerDescription = comment;
                  container.ContainerStatus = PmxContainerStatus.Open;
                  container.ContainerType = PmxContainerType.Purchase;
                  container.ExpeditionNumber = expeditionNumber;
                  container.IncidentType = reasonCode;
                  container.MaxPrice = currentPrice;
                  container.MaxWeight = (double)weight;
                  container.NumberOfPackages = (int)numOfPackages;
                  container.ShippingType = int.Parse(shippingType);
                  container.UnitCode = unitCode;
                  container.WeightUnitCode = weightUomID;


                  cprov.AddBO(container);

                  return null;
              }));


            goto Step_PrintLabel;

        #endregion

        Step_PrintLabel:
            #region Step_PrintLabel

            string reportPath = Definitions.REPORT_PATH_LABEL;
            sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintLabel), container.DocEntry, session.DeviceInstanceId, reportPath);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
            DefaultCultureInfo.Get(), BuildParamCollection(
                "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                "TitleKey", "MSG_TITLE_REPRINT_LABEL"));

            msg = WaitForMessage();

            if (msg.Name.EndsWith(".Yes"))
            {
                goto Step_PrintLabel;
            }
        }

        #endregion

        #endregion


        #region HELPERS


        /// <summary>
        /// Executes the get PMX warehouse code.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProvider oseProv = new PmxOseProvider(conn);
            return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
        }

        /// <summary>
        /// Executes the get receive all items on one purchase delivery.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static bool ExecuteGetReceiveAllItemsOnOnePurchaseDelivery(PmxDbConnection conn, object[] parameters)
        {
            PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider((SboConnection)conn);
            return prov.ReceiveAllItemsOnOnePurchaseDelivery();
        }

        /// <summary>
        /// Executes the get allow to select batches.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static bool ExecuteGetAllowToSelectBatches(PmxDbConnection conn, object[] parameters)
        {
            PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider((SboConnection)conn);

            return prov.AllowToSelectBatches();
        }

        /// <summary>
        /// Executes the check allow multiple identical log units.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxBusinessPartnerInfo ExecuteGetBusinessPartnerInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxBusinessPartnerProvider prov = new PmxBusinessPartnerProvider((SboConnection)conn);
            return prov.GetBusinessPartnerInfo((string)parameters[0]);
        }

        /// <summary>
        /// Gets the transactional packaging info.
        /// </summary>
        /// <param name="purchaseDeliveryNoteList">The purchase delivery note list.</param>
        /// <param name="batchNumber">The batch number.</param>
        /// <param name="secondBatchNumber">The second batch number.</param>
        /// <param name="bestBeforeDate">The best before date.</param>
        /// <param name="itemCode">The item code.</param>
        /// <returns></returns>
        private static Collection<ItemTransactionalPackagingTypeInfo> GetTransactionalPackagingInfo(Collection<LogisticUnitGoodsReceipt> purchaseDeliveryNoteList, string batchNumber, string secondBatchNumber, DateTime? bestBeforeDate, string itemCode)
        {
            //does it exist in the just received items?
            foreach (LogisticUnitGoodsReceipt receipt in purchaseDeliveryNoteList)
            {
                foreach (LogisticUnitItemGoodReceipt receiptItem in receipt.ItemsOnLogisticUnit)
                {
                    if (batchNumber == receiptItem.BatchNumber1 &&
                        secondBatchNumber == receiptItem.BatchNumber2 &&
                        bestBeforeDate == receiptItem.BestBeforeDate &&
                        itemCode == receiptItem.ItemCode)
                    {
                        //we found the packagingData
                        return receiptItem.ListOfPackagingTypes;
                    }
                }
            }
            return null;
        }

        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }

        /// <summary>
        /// Sets the item transactional packaging type info list.
        /// </summary>
        /// <param name="itemInfo">The item info.</param>
        /// <returns></returns>
        private Collection<ItemTransactionalPackagingTypeInfo> SetItemTransactionalPackagingTypeInfoList(PmxItemInfo itemInfo)
        {
            //get the packaging types qty from item
            Collection<ItemTransactionalPackagingTypeInfo> listOfTransactionalPackagingTypeInfos = new Collection<ItemTransactionalPackagingTypeInfo>();

            foreach (PmxItemPackagingTypeInfo packInfo in itemInfo.ListOfPackagingTypes)
            {
                ItemTransactionalPackagingTypeInfo newPackInfo = new ItemTransactionalPackagingTypeInfo();

                newPackInfo.PackagingTypeCode = packInfo.PackagingTypeCode;
                newPackInfo.Quantity = packInfo.Quantity;
                newPackInfo.ItemCode = itemInfo.ItemCode;

                listOfTransactionalPackagingTypeInfos.Add(newPackInfo);

            }

            return listOfTransactionalPackagingTypeInfos;
        }

        /// <summary>
        /// Executes the get sbo company info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static SboCompanyAdminInfo ExecuteGetSboCompanyInfo(PmxDbConnection conn, object[] parameters)
        {
            SboCompanyProvider prov = new SboCompanyProvider((SboConnection)conn);
            return prov.GetAdminInfo();
        }

        /// <summary>
		/// Executes the print label.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOsePrinter ExecutePrintLabel(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@containerDocEntry", (int)parameters[0]));

            return prov.PrintReport(reportParameters, (string)parameters[2], Definitions.NUMBER_OF_LABELS, (string)parameters[1], null, false);
        }

        #endregion

        #region BUILD QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        internal static class BuildQuery
        {
            /// <summary>
            /// Command for getting all suppliers order by name
            /// </summary>
            /// <returns>
            /// Command for getting all suppliers order by name
            /// </returns>
            public static string GetSuppliersOrderByName()
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddColumnNameWithAs(strBuilder, SboBusinessPartnerTable.Columns.CardCode.NAME, SupplierDataSet.Columns.SUPPLIER_CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddColumnNameWithAs(strBuilder, SboBusinessPartnerTable.Columns.CardName.NAME, SupplierDataSet.Columns.SUPPLIER_NAME.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboBusinessPartnerTable.Columns.CardType.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBusinessPartnerTypeColumnValues.Vendor.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboBusinessPartnerTable.Columns.CardName.NAME));
                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the existing ITRI.
            /// </summary>
            /// <param name="itemCode">The item code.</param>
            /// <param name="batchNumber">The batch number.</param>
            /// <param name="batchNumber2">The batch number2.</param>
            /// <param name="bestBeforeDate">The best before date.</param>
            /// <returns></returns>
            public static string GetExistingITRI(string itemCode, string batchNumber, string batchNumber2, DateTime? bestBeforeDate)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.InternalKey.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalPackagingTypeTable.Columns.PackagingTypeCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalPackagingTypeTable.Columns.Quantity.NAME));


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));

                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalPackagingTypeTable.Columns.ItemTransactionalInfoKey.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.InternalKey.NAME));



                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

                if (!string.IsNullOrEmpty(batchNumber))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(batchNumber, CultureInfo.InvariantCulture));
                }

                if (!string.IsNullOrEmpty(batchNumber2))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(batchNumber2, CultureInfo.InvariantCulture));
                }

                if (bestBeforeDate.HasValue)
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(bestBeforeDate.Value, CultureInfo.InvariantCulture));
                }

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalPackagingTypeTable.Columns.Quantity.NAME));


                return strBuilder.ToString();
            }

            /// <summary>
            /// 
            /// </summary>
            /// <param name="dbTool"></param>
            /// <returns></returns>
            public static string GetAllShippingType(DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.AppendFormat("SELECT CAST({0} AS NVARCHAR) AS {1}, {2} FROM {3}  ",
                    SqlCommandHelper.EscapeTableColumnName(SboShippingTypeTable.NAME, "TrnspCode"),
                    SqlCommandHelper.EscapeColumnValue(CustomerDataSet.Columns.CUSTOMER_CODE.Name),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(SboShippingTypeTable.NAME, "TrnspName", CustomerDataSet.Columns.CUSTOMER_NAME.Name),
                    SqlCommandHelper.EscapeTableName(SboShippingTypeTable.NAME));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the reasons.
            /// </summary>
            /// <returns></returns>
            public static string GetReasons()
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT  ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Code.NAME, ReasonDataSet.Columns.REASON_CODE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Name.NAME, ReasonDataSet.Columns.REASON_NAME.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.RequiresFreeText.NAME, ReasonDataSet.Columns.REASON_REQUIRES_FREE_TEXT.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxReasonTable.NAME));

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReasonTable.Columns.UseForContainer.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.AppendFormat(
              " ORDER BY CASE WHEN {0} IS NULL THEN 1 ELSE 0 END, {0} ",
               SqlCommandHelper.EscapeTableColumnName(PmxReasonTable.NAME, PmxReasonTable.Columns.SequenceNumber.NAME));

                return strBuilder.ToString();
            }

            public static string SetNumberOfPackages(int docEntry, int numOfPackages)
            {
                //update pmx_cohe set NumberOfPackages = 0 where docentry = 1
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.AppendFormat("UPDATE {0} SET {1} = {2} WHERE {3} = {4}",
                    SqlCommandHelper.EscapeTableName(PmxContainerTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxContainerTable.NAME, PmxContainerTable.Columns.NumberOfPackages.NAME),
                    SqlCommandHelper.EscapeColumnValue(numOfPackages),
                    SqlCommandHelper.EscapeTableColumnName(PmxContainerTable.NAME, PmxContainerTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeColumnValue(docEntry)
                    );

                return strBuilder.ToString();
            }
        }

        #endregion
    }
}